from datetime import datetime

from backports.datetime_fromisoformat import MonkeyPatch

# Python 3.6 compatibility
MonkeyPatch.patch_fromisoformat()


class GoodreadsEvent:
    def __init__(self, event_dict):
        self._event_dict = event_dict

    def __repr__(self):
        return self.title

    @property
    def gid(self):
        """ID for the event"""
        return int(self._event_dict["id"]["#text"])

    @property
    def title(self):
        """Title for the event"""
        return self._event_dict["title"]

    @property
    def description(self):
        """Description for the event"""
        return self._event_dict["description"]

    @property
    def link(self):
        """Link for the event"""
        return self._event_dict["link"]

    @property
    def venue(self):
        """Venue for the event"""
        return self._event_dict["venue"]

    @property
    def address(self):
        """Address for the event"""
        return self._event_dict["address"]

    @property
    def city(self):
        """City for the event"""
        return self._event_dict["city"]

    @property
    def postal_code(self):
        """Postal code for the event"""
        return self._event_dict["postal_code"]

    @property
    def state_code(self):
        """State for the event"""
        return self._event_dict["state_code"]

    @property
    def country_code(self):
        """Country code for the event"""
        return self._event_dict["country_code"]

    @property
    def access(self):
        """Is event public or private?"""
        return self._event_dict["access"]

    @property
    def event_type(self):
        """Type of the event"""
        return self._event_dict["event_type"]

    @property
    def added_by(self):
        """User id for the event creator"""
        return int(self._event_dict["user_id"]["#text"])

    @property
    def image_url(self):
        """Image URL for the event"""
        return self._event_dict["image_url"]

    @property
    def created_at(self):
        """Event created at"""
        return datetime.fromisoformat(self._event_dict["created_at"]["#text"])

    @property
    def updated_at(self):
        """Event updated at"""
        return datetime.fromisoformat(self._event_dict["updated_at"]["#text"])

    @property
    def reminder_at(self):
        """Reminder time for the event"""
        reminder = self._event_dict.get("reminder_at")
        if reminder:
            return datetime.fromisoformat(reminder.get("#text"))
        else:
            return None

    @property
    def rsvp_end_at(self):
        """RSVP for the event ends at"""
        rsvp_end_at = self._event_dict.get("rsvp_end_at")
        if rsvp_end_at:
            return datetime.fromisoformat(rsvp_end_at.get("#text"))
        else:
            return None

    @property
    def start_at(self):
        """Event starts at"""
        return datetime.fromisoformat(self._event_dict["start_at"]["#text"])

    @property
    def end_at(self):
        """Event ends at"""
        return datetime.fromisoformat(self._event_dict["end_at"]["#text"])

    @property
    def attending_count(self):
        """Number of people attending"""
        return int(self._event_dict["event_attending_count"]["#text"])

    @property
    def responses_count(self):
        """Number of people who responded to the invite"""
        return int(self._event_dict["event_responses_count"]["#text"])

    @property
    def resource(self):
        """Type and ID for the event resource"""
        return (
            self._event_dict["resource_type"],
            int(self._event_dict["resource_id"]["#text"]),
        )



from datetime import datetime

from betterreads.author import GoodreadsAuthor


class GoodreadsBook:
    def __init__(self, book_dict, client):
        self._book_dict = book_dict
        self._client = client

    def __repr__(self):
        return self.title

    @property
    def gid(self):
        """Goodreads id of the book"""
        return int(self._book_dict["id"])

    @property
    def title(self):
        """Title of the book"""
        return self._book_dict["title"]

    @property
    def authors(self):
        """Authors of the book"""
        # Goodreads API returns a list if there are more than one authors,
        # otherwise, just the OrderedDict
        if type(self._book_dict["authors"]["author"]) == list:
            return [
                GoodreadsAuthor(author_dict, self._client)
                for author_dict in self._book_dict["authors"]["author"]
            ]
        else:
            return [GoodreadsAuthor(self._book_dict["authors"]["author"], self._client)]

    @property
    def description(self):
        """Description of the book"""
        return self._book_dict["description"]

    @property
    def average_rating(self):
        """Average rating of the book"""
        return float(self._book_dict["average_rating"])

    @property
    def rating_dist(self):
        """Rating distribution of the book"""
        return self._book_dict["work"]["rating_dist"]

    @property
    def ratings_count(self):
        """Number of ratings for the book"""
        return int(self._book_dict["ratings_count"])

    @property
    def text_reviews_count(self):
        """Number of text reviews for the book"""
        return int(self._book_dict["text_reviews_count"])

    @property
    def num_pages(self):
        """Number of pages of the book"""
        return int(self._book_dict["num_pages"])

    @property
    def popular_shelves(self):
        """A count of hw many user shelves with the same name contain this book"""
        return [
            shelf_dict for shelf_dict in self._book_dict["popular_shelves"]["shelf"]
        ]

    @property
    def work(self):
        """Information on the original work"""
        return self._book_dict["work"]

    @property
    def series_works(self):
        """Return series of the book"""
        return self._book_dict["series_works"]

    @property
    def publication_date(self):
        """Publication month/day/year for the book"""
        return datetime(
            int(self._book_dict["publication_year"]),
            int(self._book_dict["publication_month"]),
            int(self._book_dict["publication_day"]),
        )

    @property
    def publisher(self):
        """Publisher for the book"""
        return self._book_dict["publisher"]

    @property
    def language_code(self):
        """Language code for the book"""
        return self._book_dict["language_code"]

    @property
    def edition_information(self):
        """Edition information of the book"""
        return self._book_dict["edition_information"]

    @property
    def image_url(self):
        """Image URL of the book"""
        return self._book_dict["image_url"]

    @property
    def small_image_url(self):
        """Small image URL of the book"""
        return self._book_dict["small_image_url"]

    @property
    def is_ebook(self):
        """Is this book an e-book?"""
        return False if self._book_dict["is_ebook"] == "false" else True

    @property
    def format(self):
        """Book format"""
        return self._book_dict["format"]

    @property
    def isbn(self):
        """ISBN of the book"""
        return self._book_dict["isbn"]

    @property
    def isbn13(self):
        """ISBN13 of the book"""
        return self._book_dict["isbn13"]

    @property
    def link(self):
        """Link for the book"""
        return self._book_dict["link"]

    @property
    def reviews_widget(self):
        """Widget for reviews in HTML"""
        return self._book_dict["reviews_widget"]

    @property
    def similar_books(self):
        """Return the list of similar books."""
        return [
            GoodreadsBook(b, self._client)
            for b in self._book_dict["similar_books"]["book"]
        ]



from datetime import datetime

from backports.datetime_fromisoformat import MonkeyPatch

from betterreads.book import GoodreadsBook
from betterreads.review import GoodreadsReview

# Python version 3.6 compatibility
MonkeyPatch.patch_fromisoformat()


class GoodreadsOwnedBook:
    def __init__(self, owned_book_dict):
        self._owned_book_dict = owned_book_dict

    @property
    def gid(self):
        """Goodreads id of the owned book"""
        return int(self._owned_book_dict["id"]["#text"])

    @property
    def book(self):
        """Book owned"""
        return GoodreadsBook(self._owned_book_dict["book"], self)

    @property
    def review(self):
        """Review for the owned book"""
        return GoodreadsReview(self._owned_book_dict["review"])

    @property
    def current_owner(self):
        """Return current owner's id"""
        return int(self._owned_book_dict["current_owner_id"]["#text"])

    @property
    def original_purchase_date(self):
        """Date of purchase"""
        date_string = self._owned_book_dict.get("original_purchase_date", {}).get(
            "#text", None
        )
        return datetime.fromisoformat(date_string) if date_string else None

    @property
    def original_purchase_location(self):
        """Purchase location"""
        return self._owned_book_dict.get("original_purchase_location", None)

    @property
    def condition(self):
        """Condition of the book"""
        return self._owned_book_dict["condition"]

    @property
    def link(self):
        """Linked for the owned book"""
        return self._owned_book_dict["link"]



class GoodreadsUserShelf:
    """
    Interface for interacting with a GoodreadsUser's shelves
    """

    def __init__(self, shelf_dict):
        self._shelf_dict = shelf_dict

    def __repr__(self):
        return self.name

    @property
    def gid(self):
        return int(self._shelf_dict.get("id", {}).get("#text"))

    @property
    def name(self):
        return self._shelf_dict.get("name")

    @property
    def exclusive(self):
        """
        A flag which describes whether a shelf is mutually exclusive with a user's other exclusive shelves
        """
        return (
            True
            if self._shelf_dict.get("exclusive_flag", {}).get("#text") == "true"
            else False
        )

    @property
    def count(self):
        """
        Number of books on a GoodreadsUserShelf
        """
        return int(self._shelf_dict.get("book_count", {}).get("#text"))

    @property
    def sticky(self):
        """
        Describes whether a user has designated a shelf as "sticky" in the Goodreads interface
        """
        return (
            True if self._shelf_dict.get("sticky", {}).get("#text") == "true" else False
        )

    @property
    def description(self):
        return self._shelf_dict.get("description", {}).get("#text")

    @property
    def featured(self):
        """
        A flag which describes whether a shelf is a user's featured shelf
        """
        return (
            True
            if self._shelf_dict.get("featured", {}).get("#text") == "true"
            else False
        )



import collections

from betterreads.group import GoodreadsGroup
from betterreads.owned_book import GoodreadsOwnedBook
from betterreads.review import GoodreadsReview
from betterreads.user_shelf import GoodreadsUserShelf


class GoodreadsUser:
    def __init__(self, user_dict, client):
        self._user_dict = user_dict
        self._client = client  # for later queries

    def __repr__(self):
        if self.user_name:
            return self.user_name
        else:
            return self.gid

    @property
    def gid(self):
        """Goodreads ID for the user"""
        return int(self._user_dict["id"])

    @property
    def user_name(self):
        """Goodreads handle of the user"""
        return self._user_dict["user_name"]

    @property
    def name(self):
        """Name of the user"""
        return self._user_dict["name"]

    @property
    def link(self):
        """URL for user profile"""
        return self._user_dict["link"]

    @property
    def image_url(self):
        """URL of user image"""
        return self._user_dict["image_url"]

    @property
    def small_image_url(self):
        """URL of user image (small)"""
        return self._user_dict["small_image_url"]

    def list_groups(self, page=1):
        """List groups for the user. If there are more than 30 groups, get them
        page by page."""
        try:
            resp = self._client.request("group/list/%s.xml" % self.gid, {"page": page})
            groups = [
                GoodreadsGroup(group_dict)
                for group_dict in resp["groups"]["list"]["group"]
            ]
        except KeyError:
            groups = []
        return groups

    def owned_books(self, page=1):
        """Return the list of books owned by the user"""
        try:
            resp = self._client.session.get(
                "owned_books/user", {"page": page, "format": "xml", "id": self.gid}
            )
            owned_books_resp = resp["owned_books"]["owned_book"]
            # If there's only one owned book returned, put it in a list.
            if type(owned_books_resp) == collections.OrderedDict:
                owned_books_resp = [owned_books_resp]
            owned_books = [GoodreadsOwnedBook(d) for d in owned_books_resp]
        except KeyError:
            owned_books = []
        return owned_books

    def reviews(self, page=1):
        """Get all books and reviews on user's shelves"""
        resp = self._client.request(
            "/review/list.xml", {"v": 2, "id": self.gid, "page": page}
        )
        return [GoodreadsReview(r) for r in resp["reviews"]["review"]]

    def shelves(self, page=1):
        """Get the user's shelves. This method gets shelves only for users with
        public profile"""
        resp = self._client.request(
            "shelf/list.xml", {"user_id": self.gid, "page": page}
        )
        return [GoodreadsUserShelf(s) for s in resp["shelves"]["user_shelf"]]

    def per_shelf_reviews(self, page=1, per_page=200, shelf_name="read"):
        """Get all books and reviews on a user's particular shelf"""
        total = 1
        all_reviews = []
        while len(all_reviews) < total:
            resp = self._client.request(
                "/review/list.xml",
                {
                    "v": 2,
                    "id": self.gid,
                    "page": page,
                    "shelf": shelf_name,
                    "per_page": per_page,
                },
            )
            reviews = [GoodreadsReview(r) for r in resp["reviews"]["review"]]
            all_reviews.extend(reviews)
            page += 1
            total = int(resp["reviews"]["@total"])
        return all_reviews



from datetime import datetime


class GoodreadsAuthor:
    def __init__(self, author_dict, client):
        self._author_dict = author_dict
        self._client = client

    def __repr__(self):
        return self.name

    @property
    def gid(self):
        """Goodreads id of the author"""
        return int(self._author_dict["id"])

    @property
    def name(self):
        """Author name"""
        return self._author_dict["name"]

    @property
    def about(self):
        """About the author"""
        return self._author_dict["about"]

    @property
    def books(self):
        """Books of the author"""
        # Goodreads API returns a list if there are more than one books, otherwise,
        # just the dict.
        from .book import GoodreadsBook

        if type(self._author_dict["books"]["book"]) == list:
            return [
                GoodreadsBook(book_dict, self._client)
                for book_dict in self._author_dict["books"]["book"]
            ]
        else:
            return [GoodreadsBook(self._author_dict["books"]["book"], self._client)]

    @property
    def born_at(self):
        """Author date of birth"""
        born_date = self._author_dict["born_at"]
        return datetime.strptime(born_date, "%Y/%m/%d")

    @property
    def died_at(self):
        """Author date of death"""
        died_date = self._author_dict["died_at"]
        return datetime.strptime(died_date, "%Y/%m/%d")

    @property
    def fans_count(self):
        """Number of fans"""
        return int(self._author_dict["fans_count"]["#text"])

    @property
    def gender(self):
        """Author gender"""
        return self._author_dict["gender"]

    @property
    def hometown(self):
        """Author's hometown"""
        return self._author_dict["hometown"]

    @property
    def link(self):
        """Link for author page"""
        return self._author_dict["link"]

    @property
    def image_url(self):
        """Image URL"""
        return self._author_dict["image_url"]

    @property
    def small_image_url(self):
        """Small image URL"""
        return self._author_dict["small_image_url"]

    @property
    def influences(self):
        """Influenced by"""
        return self._author_dict["influences"]

    @property
    def user(self):
        """Goodreads user profile of the author"""
        from betterreads.user import GoodreadsUser

        goodreads_user = None
        if "user" in self._author_dict:
            goodreads_user = GoodreadsUser(
                self._author_dict["user"]["id"]["#text"], self._client
            )
        return goodreads_user

    @property
    def works_count(self):
        """Author number of works"""
        return int(self._author_dict["works_count"])



import requests
import xmltodict
import json


class GoodreadsRequestException(Exception):
    def __init__(self, error_msg, url):
        self.error_msg = error_msg
        self.url = url

    def __str__(self):
        return self.url, ":", self.error_msg


class GoodreadsRequest:
    def __init__(self, client, path, query_dict, req_format="xml"):
        """Initialize request object."""
        self.params = query_dict
        self.params.update(client.query_dict)
        self.host = client.base_url
        self.path = path
        self.req_format = req_format

    def request(self):
        resp = requests.get(self.host + self.path, params=self.params)
        if resp.status_code != 200:
            raise GoodreadsRequestException(resp.reason, self.path)
        if self.req_format == "xml":
            data_dict = xmltodict.parse(resp.content, dict_constructor=dict)
            return data_dict["GoodreadsResponse"]
        elif self.req_format == "json":
            return json.loads(resp.content)
        else:
            raise Exception("Invalid format")



from datetime import datetime


class GoodreadsReview:
    def __init__(self, review_dict):
        self._review_dict = review_dict

    def __repr__(self):
        return "review [%s]" % self.gid

    @property
    def gid(self):
        """Goodreads id for the review"""
        return int(self._review_dict["id"])

    @property
    def book(self):
        """Book that the review belongs to"""
        return self._review_dict["book"]

    @property
    def rating(self):
        """Rating of the book"""
        return int(self._review_dict["rating"])

    @property
    def shelves(self):
        """Shelves for the book"""
        try:
            return [shelf["@name"] for shelf in self._review_dict["shelves"]["shelf"]]
        # In some cases it appears the Goodreads API returns a response
        # where 'shelf' object is not a list but rather a single shelf.
        # In this case the Exception "TypeError: string indices must be
        # integers" is raised.
        except TypeError:
            return [self._review_dict["shelves"]["shelf"]["@name"]]

    @property
    def recommended_for(self):
        """Book recommended for"""
        return self._review_dict["recommended_for"]

    @property
    def recommended_by(self):
        """Book recommended by"""
        return self._review_dict["recommended_by"]

    @property
    def started_at(self):
        """Book started at"""
        return datetime.strptime(
            self._review_dict["started_at"], "%a %b %d %H:%M:%S %z %Y"
        )

    @property
    def read_at(self):
        """Book read at"""
        return datetime.strptime(
            self._review_dict["read_at"], "%a %b %d %H:%M:%S %z %Y"
        )

    @property
    def body(self):
        """Body for the review"""
        return self._review_dict["body"]

    @property
    def comments_count(self):
        """Number of comments to this review"""
        return int(self._review_dict["comments_count"])

    @property
    def url(self):
        """URL for this comment"""
        return self._review_dict["url"]

    @property
    def owned(self):
        """Is the book owned by this user?"""
        return False if self._review_dict["owned"] == "0" else True



from datetime import datetime


class GoodreadsGroup:
    """Goodreads Group class implementation"""

    #   TODO: Add moderators property
    #   TODO: convert last_activity_at to datetime
    def __init__(self, group_dict):
        self._group_dict = group_dict

    def __repr__(self):
        return self.title

    @property
    def gid(self):
        """Goodreads id for the group"""
        return int(self._group_dict["id"])

    @property
    def title(self):
        """Title for the group"""
        return self._group_dict["title"]

    @property
    def description(self):
        """Description of the group"""
        return self._group_dict["description"]

    @property
    def category(self):
        """Category of the group"""
        return self._group_dict["category"]

    @property
    def subcategory(self):
        """Sub-category of the group"""
        return self._group_dict["subcategory"]

    @property
    def rules(self):
        """Group rules"""
        return self._group_dict["rules"]

    @property
    def image_url(self):
        """Image URL for the group"""
        return self._group_dict["image_url"]

    @property
    def last_activity_at(self):
        """Time of the last activity in the group"""
        return datetime.strptime(
            self._group_dict["last_activity_at"], "%a %b %d %H:%M:%S %z %Y"
        )

    @property
    def access(self):
        """Is the group public or private?"""
        return self._group_dict["access"]

    @property
    def users_count(self):
        """Number of users in the group"""
        return int(self._group_dict["group_users_count"])

    @property
    def members(self):
        """Members of the group"""
        return self._group_dict["members"]["group_user"]



from datetime import datetime

from betterreads.user import GoodreadsUser


class GoodreadsComment:
    """Goodreads comment class"""

    def __init__(self, comment_dict):
        self._comment_dict = comment_dict

    @property
    def gid(self):
        """Goodreads id of the comment"""
        return int(self._comment_dict["id"])

    @property
    def body(self):
        """Body of the comment"""
        return self._comment_dict["body"]

    @property
    def user(self):
        """User that made the comment"""
        return GoodreadsUser(self._comment_dict["user"], self)

    @property
    def created_at(self):
        """Comment created at"""
        return datetime.strptime(
            self._comment_dict["created_at"], "%a %b %d %H:%M:%S %z %Y"
        )

    @property
    def updated_at(self):
        """Comment updated at"""
        return datetime.strptime(
            self._comment_dict["updated_at"], "%a %b %d %H:%M:%S %z %Y"
        )



from os import path

from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()


setup(
    name="betterreads",
    description="Python 3 wrapper for Goodreads API",
    long_description=long_description,
    url="https://github.com/thejessleigh/betterreads",
    author="Jess Unrein",
    author_email="j.l.unrein@gmail.com",
    packages=["betterreads"],
    version="0.4.2",
    install_requires=[
        "backports-datetime-fromisoformat",
        "rauth",
        "requests",
        "xmltodict",
    ],
    license="MIT",
    keywords="goodreads API",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    long_description_content_type="text/x-rst",
)



import webbrowser

from betterreads.author import GoodreadsAuthor
from betterreads.book import GoodreadsBook
from betterreads.comment import GoodreadsComment
from betterreads.event import GoodreadsEvent
from betterreads.group import GoodreadsGroup
from betterreads.owned_book import GoodreadsOwnedBook
from betterreads.request import GoodreadsRequest
from betterreads.review import GoodreadsReview
from betterreads.session import GoodreadsSession
from betterreads.user import GoodreadsUser


class GoodreadsClientException(Exception):
    def __init__(self, error_msg):
        self.error_msg = error_msg

    def __str__(self):
        return self.error_msg


class GoodreadsClient:
    base_url = "https://www.goodreads.com/"

    def __init__(self, client_key, client_secret):
        """Initialize the client"""
        self.client_key = client_key
        self.client_secret = client_secret

    @property
    def query_dict(self):
        return {"key": self.client_key}

    def authenticate(self, access_token=None, access_token_secret=None):
        """Authenticate client to query requiring authorization"""
        self.session = GoodreadsSession(
            self.client_key, self.client_secret, access_token, access_token_secret
        )
        if access_token and access_token_secret:
            self.session.oauth_resume()
        else:
            url = self.session.oauth_init()
            webbrowser.open(url)
            while input("Have you authorized me? (y/n)") != "y":
                pass
            self.session.oauth_finalize()

    def auth_user(self):
        """Return user who authorized OAuth"""
        if not hasattr(self, "session"):
            raise GoodreadsClientException("No authenticated session")
        resp = self.request_oauth("api/auth_user", {})
        user_id = resp["user"]["@id"]
        return self.user(user_id)

    def request(self, *args, **kwargs):
        """Create a GoodreadsRequest object and make that request"""
        req = GoodreadsRequest(self, *args, **kwargs)
        return req.request()

    def request_oauth(self, *args, **kwargs):
        resp = self.session.get(*args, **kwargs)
        return resp

    def user(self, user_id=None, username=None):
        """Get info about a member by id or username.

        If user_id or username not provided, the function returns the
        authorized user.
        """
        if not (user_id or username):
            return self.auth_user()
        resp = self.request("user/show", {"id": user_id, "username": username})
        return GoodreadsUser(resp["user"], self)

    def author(self, author_id):
        """Get info about an author"""
        resp = self.request("author/show", {"id": author_id})
        return GoodreadsAuthor(resp["author"], self)

    def find_author(self, author_name):
        """Find an author by name"""
        resp = self.request("api/author_url/%s" % author_name, {})
        return self.author(resp["author"]["@id"]) if "author" in resp else None

    def book(self, book_id=None, isbn=None):
        """Get info about a book"""
        if book_id:
            resp = self.request("book/show", {"id": book_id})
            return GoodreadsBook(resp["book"], self)
        elif isbn:
            resp = self.request("book/isbn", {"isbn": isbn})
            return GoodreadsBook(resp["book"], self)
        else:
            raise GoodreadsClientException("book id or isbn required")

    def search_books(self, q, page=1, search_field="all"):
        """Get the most popular books for the given query. This will search all
        books in the title/author/genre fields and show matches, sorted by
        popularity on Goodreads.
        :param q: query text
        :param page: which page to return (default 1)
        :param search_fields: field to search, one of 'title', 'author' or
        'genre' (default is 'all')
        """
        resp = self.request(
            "search/index.xml", {"q": q, "page": page, "search[field]": search_field}
        )
        works = resp["search"]["results"]["work"]
        # If there's only one work returned, put it in a list.
        if type(works) == dict:
            works = [works]
        return [self.book(work["best_book"]["id"]["#text"]) for work in works]

    def search_books_total_pages(self, q, page=1, search_field="all"):
        """Get the total of pages from the search. This will search all
        books in the title/author/ISBN fields and show matches, sorted by
        popularity on Goodreads.
        :param q: query text
        :param page: which page to return (default 1)
        :param search_fields: field to search, one of 'title', 'author' or
        'genre' (default is 'all')
        """
        resp = self.request(
            "search/index.xml", {"q": q, "page": page, "search[field]": search_field}
        )
        total = resp["search"]["total-results"]
        books_per_page = 20

        # convert to integer plus 1 in case that the result it's a float
        pages = int(int(total) / books_per_page) + 2
        return pages

    def search_books_all_pages(self, q, page=1, search_field="all"):
        """Get all books for the given query. This will search all
        books in the title/author/ISBN fields and show matches, sorted by
        popularity on Goodreads.
        :param q: query text
        :param page: which page to return (default 1)
        :param search_fields: field to search, one of 'title', 'author' or
        'genre' (default is 'all')
        """
        pages = self.search_books_total_pages(q=q, search_field=search_field)
        books = [self.search_books(q, p, search_field) for p in list(range(1, pages))]
        outputlist = []
        for i in books:
            outputlist.extend(i)
        return outputlist

    def group(self, group_id):
        """Get info about a group"""
        resp = self.request("group/show", {"id": group_id})
        return GoodreadsGroup(resp["group"])

    def owned_book(self, owned_book_id):
        """Get info about an owned book, given its id"""
        resp = self.session.get(
            "owned_books/show/%s" % owned_book_id, {"format": "xml"}
        )
        return GoodreadsOwnedBook(resp["owned_book"]["owned_book"])

    def find_groups(self, query, page=1):
        """Find a group based on the query"""
        resp = self.request("group/search.xml", {"q": query, "page": page})
        groups_dicts = resp["groups"]["list"]["group"]
        return [GoodreadsGroup(g) for g in groups_dicts]

    def book_review_stats(self, isbns):
        """Get review statistics for books given a list of ISBNs"""
        resp = self.request(
            "book/review_counts.json", {"isbns": ",".join(isbns)}, req_format="json"
        )
        return resp["books"]

    def list_comments(self, comment_type, resource_id, page=1):
        """List comments on a subject.

        comment_type should be one of 'author_blog_post', 'blog',
        'book_news_post', 'chapter', 'comment', 'community_answer',
        'event_response', 'fanship', 'friend', 'giveaway', 'giveaway_request',
        'group_user', 'interview', 'librarian_note', 'link_collection', 'list',
        'owned_book', 'photo', 'poll', 'poll_vote', 'queued_item', 'question',
        'question_user_stat', 'quiz', 'quiz_score', 'rating', 'read_status',
        'recommendation', 'recommendation_request', 'review', 'topic', 'user',
        'user_challenge', 'user_following', 'user_list_challenge',
        'user_list_vote', 'user_quote', 'user_status', 'video'.
        """
        resp = self.request(
            "%s/%s/comments" % (comment_type, resource_id), {"format": "xml"}
        )
        return [
            GoodreadsComment(comment_dict)
            for comment_dict in resp["comments"]["comment"]
        ]

    def list_events(self, postal_code):
        """Show events near a location specified with the postal code"""
        resp = self.request("event/index.xml", {"search[postal_code]": postal_code})
        return [GoodreadsEvent(e) for e in resp["events"]["event"]]

    def recent_reviews(self):
        """Get the recent reviews from all members"""
        resp = self.request("/review/recent_reviews.xml", {})
        return [GoodreadsReview(r) for r in resp["reviews"]["review"]]

    def review(self, review_id):
        """Get a review"""
        resp = self.request("/review/show.xml", {"id": review_id})
        return GoodreadsReview(resp["review"])
